@@ -1,4 +1,4 @@
-from cocaine import concurrent
+from cocaine.futures import chain
 from cocaine.tools import actions
 from cocaine.tools.actions import CocaineConfigReader, log
 from cocaine.tools.printer import printer
@@ -29,7 +29,7 @@ class Upload(Specific):
         if not self.profile:
             raise ValueError('Please specify profile file path')
 
-    @concurrent.engine
+    @chain.source
     def execute(self):
         with printer('Loading profile'):
             profile = CocaineConfigReader.load(self.profile)
@@ -38,8 +38,8 @@ class Upload(Specific):
 
 
 class Remove(Specific):
-    @concurrent.engine
+    @chain.source
     def execute(self):
         log.info('Removing "%s"... ', self.name)
         yield self.storage.remove('profiles', self.name)
-        log.info('OK')
\ No newline at end of file
+        log.info('OK')
