@@ -5,8 +5,8 @@ import sys
 
 from opster import Dispatcher
 
-from cocaine.services import Locator, Service
-from tornado.ioloop import IOLoop
+from cocaine.asio.service import Locator, Service
+from cocaine.logging.hanlders import ColoredFormatter, interactiveEmit
 from cocaine.tools.actions import proxy
 from cocaine.tools.cli import Executor
 from cocaine.tools.error import Error as ToolsError
@@ -19,63 +19,6 @@ DEFAULT_HOST = 'localhost'
 DEFAULT_PORT = 10053
 
 
-BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
-RESET_SEQ = "\033[0m"
-COLOR_SEQ = "\033[1;%dm"
-BOLD_SEQ = "\033[1m"
-
-
-COLORS = {
-    'DEBUG': WHITE,
-    'INFO': GREEN,
-    'WARNING': YELLOW,
-    'CRITICAL': YELLOW,
-    'ERROR': RED
-}
-
-
-class ColoredFormatter(logging.Formatter):
-    def __init__(self, msg, colored=True):
-        logging.Formatter.__init__(self, msg)
-        self.colored = colored
-
-    def format(self, record):
-        levelname = record.levelname
-        if self.colored and levelname in COLORS:
-            record.msg = COLOR_SEQ % (30 + COLORS[levelname]) + str(record.msg) + RESET_SEQ
-        return logging.Formatter.format(self, record)
-
-
-def interactiveEmit(self, record):
-    # Monkey patch Emit function to avoid new lines between records
-    try:
-        if str(record.msg).endswith('... '):
-            fs = '%s'
-        else:
-            fs = '%s\n'
-        msg = self.format(record)
-        stream = self.stream
-        if not hasattr(logging, '_unicode') or not logging._unicode:  # if no unicode support...
-            stream.write(fs % msg)
-        else:
-            try:
-                if isinstance(msg, unicode) and getattr(stream, 'encoding', None):
-                    ufs = fs.decode(stream.encoding)
-                    try:
-                        stream.write(ufs % msg)
-                    except UnicodeEncodeError:
-                        stream.write((ufs % msg).encode(stream.encoding))
-                else:
-                    stream.write(fs % msg)
-            except UnicodeError:
-                stream.write(fs % msg.encode("UTF-8"))
-        self.flush()
-    except (KeyboardInterrupt, SystemExit):
-        raise
-    except:
-        self.handleError(record)
-
-
 class Global(object):
     options = [
         ('h', 'host', DEFAULT_HOST, 'hostname'),
@@ -120,18 +63,6 @@ class Global(object):
             log.propagate = False
             log.addHandler(ch)
 
-    def run_sync(self, func):
-        result_future = [None]
-
-        def handle_result(future):
-            result_future[0] = future
-            IOLoop.current().stop()
-
-        df = func()
-        df.add_callback(handle_result)
-        IOLoop.current().start()
-        result_future[0].get()
-
     @property
     def locator(self):
         if self._locator:
@@ -139,7 +70,7 @@ class Global(object):
         else:
             try:
                 locator = Locator()
-                self.run_sync(lambda: locator.connect(self.host, self.port, self.timeout))
+                locator.connect(self.host, self.port, self.timeout, blocking=True)
                 self._locator = locator
                 return locator
             except Exception as err:
@@ -147,8 +78,8 @@ class Global(object):
 
     def getService(self, name):
         try:
-            service = Service(name)
-            self.run_sync(lambda: service.connect_through_locator(self.locator, self.timeout))
+            service = Service(name, blockingConnect=False)
+            service.connectThroughLocator(self.locator, self.timeout, blocking=True)
             return service
         except Exception as err:
             raise ToolsError(err)
@@ -171,15 +102,17 @@ def middleware(func):
 
 
 d = Dispatcher(globaloptions=Global.options, middleware=middleware)
+appDispatcher = Dispatcher(globaloptions=Global.options, middleware=middleware)
 
 
 class dispatcher:
-    app = Dispatcher(globaloptions=Global.options, middleware=middleware)
-    profile = Dispatcher(globaloptions=Global.options, middleware=middleware)
-    runlist = Dispatcher(globaloptions=Global.options, middleware=middleware)
-    crashlog = Dispatcher(globaloptions=Global.options, middleware=middleware)
     group = Dispatcher(globaloptions=Global.options, middleware=middleware)
-    proxy = Dispatcher()
+
+
+profileDispatcher = Dispatcher(globaloptions=Global.options, middleware=middleware)
+runlistDispatcher = Dispatcher(globaloptions=Global.options, middleware=middleware)
+crashlogDispatcher = Dispatcher(globaloptions=Global.options, middleware=middleware)
+proxyDispatcher = Dispatcher()
 
 
 @d.command()
@@ -214,7 +147,7 @@ def call(options,
     })
 
 
-@dispatcher.app.command(name='list')
+@appDispatcher.command(name='list')
 def app_list(options):
     """Show installed applications list."""
     options.executor.executeAction('app:list', **{
@@ -222,7 +155,7 @@ def app_list(options):
     })
 
 
-@dispatcher.app.command(name='view', usage='--name=NAME')
+@appDispatcher.command(usage='--name=NAME', name='view')
 def app_view(options,
              name=('n', '', 'application name')):
     """Show manifest context for application.
@@ -235,7 +168,7 @@ def app_view(options,
     })
 
 
-@dispatcher.app.command(name='upload', usage='[PATH] [--name=NAME] [--manifest=MANIFEST] [--package=PACKAGE]')
+@appDispatcher.command(name='upload', usage='[PATH] [--name=NAME] [--manifest=MANIFEST] [--package=PACKAGE]')
 def app_upload(options,
                path=None,
                name=('n', '', 'application name'),
@@ -297,7 +230,7 @@ def app_upload(options,
         })
 
 
-@dispatcher.app.command(name='remove')
+@appDispatcher.command(name='remove')
 def app_remove(options,
                name=('n', '', 'application name')):
     """Remove application from storage.
@@ -310,7 +243,7 @@ def app_remove(options,
     })
 
 
-@dispatcher.app.command(name='start')
+@appDispatcher.command(name='start')
 def app_start(options,
               name=('n', '', 'application name'),
               profile=('r', '', 'profile name')):
@@ -325,7 +258,7 @@ def app_start(options,
     })
 
 
-@dispatcher.app.command(name='pause')
+@appDispatcher.command(name='pause')
 def app_pause(options,
               name=('n', '', 'application name')):
     """Stop application.
@@ -338,7 +271,7 @@ def app_pause(options,
     })
 
 
-@dispatcher.app.command(name='stop')
+@appDispatcher.command(name='stop')
 def app_stop(options,
              name=('n', '', 'application name')):
     """Stop application."""
@@ -348,7 +281,7 @@ def app_stop(options,
     })
 
 
-@dispatcher.app.command(name='restart')
+@appDispatcher.command(name='restart')
 def app_restart(options,
                 name=('n', '', 'application name'),
                 profile=('r', '', 'profile name')):
@@ -366,7 +299,7 @@ def app_restart(options,
     })
 
 
-@dispatcher.app.command()
+@appDispatcher.command()
 def check(options,
           name=('n', '', 'application name')):
     """Checks application status."""
@@ -378,7 +311,7 @@ def check(options,
     })
 
 
-@dispatcher.profile.command(name='list')
+@profileDispatcher.command(name='list')
 def profile_list(options):
     """Show installed profiles."""
     options.executor.executeAction('profile:list', **{
@@ -386,7 +319,7 @@ def profile_list(options):
     })
 
 
-@dispatcher.profile.command(name='view')
+@profileDispatcher.command(name='view')
 def profile_view(options,
                  name=('n', '', 'profile name')):
     """Show profile configuration context."""
@@ -396,7 +329,7 @@ def profile_view(options,
     })
 
 
-@dispatcher.profile.command(name='upload')
+@profileDispatcher.command(name='upload')
 def profile_upload(options,
                    name=('n', '', 'profile name'),
                    profile=('', '', 'path to profile file')):
@@ -408,7 +341,7 @@ def profile_upload(options,
     })
 
 
-@dispatcher.profile.command(name='remove')
+@profileDispatcher.command(name='remove')
 def profile_remove(options,
                    name=('n', '', 'profile name')):
     """Remove profile from the storage."""
@@ -418,7 +351,7 @@ def profile_remove(options,
     })
 
 
-@dispatcher.runlist.command(name='list')
+@runlistDispatcher.command(name='list')
 def runlist_list(options):
     """Show uploaded runlists."""
     options.executor.executeAction('runlist:list', **{
@@ -426,7 +359,7 @@ def runlist_list(options):
     })
 
 
-@dispatcher.runlist.command(name='view')
+@runlistDispatcher.command(name='view')
 def runlist_view(options,
                  name=('n', '', 'name')):
     """Show configuration context for runlist."""
@@ -436,7 +369,7 @@ def runlist_view(options,
     })
 
 
-@dispatcher.runlist.command(name='upload')
+@runlistDispatcher.command(name='upload')
 def runlist_upload(options,
                    name=('n', '', 'name'),
                    runlist=('', '', 'path to the runlist configuration json file')):
@@ -448,7 +381,7 @@ def runlist_upload(options,
     })
 
 
-@dispatcher.runlist.command(name='create')
+@runlistDispatcher.command(name='create')
 def runlist_create(options,
                    name=('n', '', 'name')):
     """Create runlist and upload it into the storage."""
@@ -458,7 +391,7 @@ def runlist_create(options,
     })
 
 
-@dispatcher.runlist.command(name='remove')
+@runlistDispatcher.command(name='remove')
 def runlist_remove(options,
                    name=('n', '', 'name')):
     """Remove runlist from the storage."""
@@ -468,7 +401,7 @@ def runlist_remove(options,
     })
 
 
-@dispatcher.runlist.command(name='add-app')
+@runlistDispatcher.command(name='add-app')
 def runlist_add_app(options,
                     name=('n', '', 'runlist name'),
                     app=('', '', 'application name'),
@@ -487,7 +420,20 @@ def runlist_add_app(options,
     })
 
 
-@dispatcher.crashlog.command(name='status')
+@runlistDispatcher.command(name='remove-app')
+def runlist_remove_app(options,
+                       name=('n', '', 'runlist name'),
+                       app=('', '', 'application name')):
+    """Remove specified application from the runlist.
+    """
+    options.executor.executeAction('runlist:remove-app', **{
+        'storage': options.getService('storage'),
+        'name': name,
+        'app': app
+    })
+
+
+@crashlogDispatcher.command(name='status')
 def crashlog_status(options):
     """Show crashlog status.
     """
@@ -495,7 +441,8 @@ def crashlog_status(options):
         'storage': options.getService('storage'),
     })
 
-@dispatcher.crashlog.command(name='list')
+
+@crashlogDispatcher.command(name='list')
 def crashlog_list(options,
                   name=('n', '', 'name')):
     """Show crashlogs list for application.
@@ -508,7 +455,7 @@ def crashlog_list(options,
     })
 
 
-@dispatcher.crashlog.command(name='view')
+@crashlogDispatcher.command(name='view')
 def crashlog_view(options,
                   name=('n', '', 'name'),
                   timestamp=('t', '', 'timestamp')):
@@ -520,7 +467,7 @@ def crashlog_view(options,
     })
 
 
-@dispatcher.crashlog.command(name='remove')
+@crashlogDispatcher.command(name='remove')
 def crashlog_remove(options,
                     name=('n', '', 'name'),
                     timestamp=('t', '', 'timestamp')):
@@ -532,7 +479,7 @@ def crashlog_remove(options,
     })
 
 
-@dispatcher.crashlog.command(name='removeall')
+@crashlogDispatcher.command(name='removeall')
 def crashlog_removeall(options,
                        name=('n', '', 'name')):
     """Remove all crashlogs for application from the storage."""
@@ -644,7 +591,7 @@ def group_pop(options,
 DEFAULT_COCAINE_PROXY_PID_FILE = '/var/run/cocaine-python-proxy.pid'
 
 
-@dispatcher.proxy.command()
+@proxyDispatcher.command()
 def start(port=('', 8080, 'server port'),
           count=('', 0, 'server subprocess count (0 means optimal for current CPU count)'),
           config=('', '/etc/cocaine/cocaine-tornado-proxy.conf', 'path to the configuration file'),
@@ -665,7 +612,7 @@ def start(port=('', 8080, 'server port'),
         logging.getLogger('cocaine.tools').error('Cocaine tool error - %s', err)
 
 
-@dispatcher.proxy.command()
+@proxyDispatcher.command()
 def stop(pidfile=('', DEFAULT_COCAINE_PROXY_PID_FILE, 'pidfile')):
     """Stop embedded cocaine proxy.
     """
@@ -678,7 +625,7 @@ def stop(pidfile=('', DEFAULT_COCAINE_PROXY_PID_FILE, 'pidfile')):
         logging.getLogger('cocaine.tools').error('Cocaine tool error - %s', err)
 
 
-@dispatcher.proxy.command()
+@proxyDispatcher.command()
 def status(pidfile=('', DEFAULT_COCAINE_PROXY_PID_FILE, 'pidfile')):
     """Show embedded cocaine proxy status.
     """
@@ -691,9 +638,9 @@ def status(pidfile=('', DEFAULT_COCAINE_PROXY_PID_FILE, 'pidfile')):
         logging.getLogger('cocaine.tools').error('Cocaine tool error - %s', err)
 
 
-d.nest('app', dispatcher.app, 'application commands')
-d.nest('profile', dispatcher.profile, 'profile commands')
-d.nest('runlist', dispatcher.runlist, 'runlist commands')
-d.nest('crashlog', dispatcher.crashlog, 'crashlog commands')
+d.nest('app', appDispatcher, 'application commands')
+d.nest('profile', profileDispatcher, 'profile commands')
+d.nest('runlist', runlistDispatcher, 'runlist commands')
+d.nest('crashlog', crashlogDispatcher, 'crashlog commands')
 d.nest('group', dispatcher.group, 'routing group commands')
-d.nest('proxy', dispatcher.proxy, 'cocaine proxy commands')
+d.nest('proxy', proxyDispatcher, 'cocaine proxy commands')
