@@ -4,4 +4,4 @@ __author__ = 'Evgeny Safronov <division494@gmail.com>'
 APPS_TAGS = ('app',)
 RUNLISTS_TAGS = ('runlist',)
 PROFILES_TAGS = ('profile',)
-GROUPS_TAGS = ('group',)
\ No newline at end of file
+GROUPS_TAGS = ('group', 'active')
